<?php
/**
 *
 * Auther: heyu
 * DateTime: 2019-08-20 15:37
 *
 */
namespace kwy\tp5util\middleware;

use kwy\tp5util\apiuser\exception\ApiParameterException;
use app\common\apiuser\model\ApiUser;
use kwy\tp5util\annotation\PropertyAnnotationTrait;
use think\exception\ClassNotFoundException;
use think\Request;

/**
 * Class Api
 * @package app\api\middleware
 * @property \kwy\tp5util\apiuser\lib\Md5Sign $md5SignLib
 * @property \kwy\tp5util\apiuser\model\ApiUser $apiUserModel
 * @property \kwy\tp5util\apiuser\model\ApiAuth $apiAuthModel
 */
class Api
{
    use PropertyAnnotationTrait;

    private $debug_info;

    private $return=['errcode'=>'0','errmsg'=>''];

    /**
     * @param $request
     * @param \Closure $next
     * @param $name
     * @return mixed
     */
    public function handle(Request $request, \Closure $next, $name)
    {
        $begin_time=microtime(true);
        if (($request->isGet()&&$request->action()=='index')||strpos(strtolower($request->server('HTTP_USER_AGENT')),'php yar rpc') !== false){
            return $next($request);
        }
        try{
            $this->checkAppUser($request);
        }catch (\Exception $e){
            $this->return['errcode']='NO_AUTH';
            $this->return['errmsg']=$e->getMessage();
            if (env('app_env')=='dev'){
                $this->return['debug_sign']=$this->md5SignLib->getLastTrueSign();
            }
            $this->return['timestamp']=time();
            return response($this->return,200,[],"json");
        }

        try{
            $service=app("app\\api\\interfaces\\".$request->controller());
            $method=new \ReflectionMethod($service,$request->action());
            $paramsList=$method->getParameters();
            $paramsArray=[];
            foreach($paramsList as $k=>$v){
                $param=$request->param($v->getName());
                if ($param===null){
                    if(!$v->isDefaultValueAvailable()){
                        throw new ApiParameterException('参数缺失:'.$v->getName());
                    }else{
                        $default_value=$v->getDefaultValue();
                        $paramsArray[]=$default_value;
                    }
                }else{
                    $paramsArray[]=$param;
                }
            }
            #begin region全局参数处理 比如公共分页逻辑
            #end region
            $ret=$method->invokeArgs($service,$paramsArray);
            $this->return['result']=$ret;
        }catch (ClassNotFoundException $te){
            $this->return['errcode']='FUNCTION_ERROR';
            $this->return['errmsg']='ERROR SERVICE';
            if ($this->isSandBox()){
                dump($te);
            }
        }catch (\ReflectionException $re){
            $this->return['errcode']='FUNCTION_ERROR';
            $this->return['errmsg']='ERROR ACTION';
        }catch(ApiParameterException $ae){
            $this->return['errcode']='PARAMS_ERROR';
            $this->return['errmsg']=$ae->getMessage();
        }catch (\Exception $e){
            $this->return['errcode']='BIZ_ERROR';
            $this->return['errmsg']=$e->getMessage();
            $this->return['source']=$e->getFile().':'.$e->getLine();
            if ($this->isSandBox()){
                dump($e);
            }
        }catch (\Error $e){
            $this->return['errcode']='SYSTEM_ERROR';
            $this->return['errmsg']=$e->getMessage();
        }
        return response($this->return,200,[],"json");
    }


    /**
     * @param Request $request
     * @throws \Exception
     */
    private function checkAppUser($request){
        $paramsNeedSign=$request->param();
        if ($this->isSandBox()){
            $params=['appid'=>'123','sign'=>'','timestamp'=>time(),'nonce_str'=>'xxx'];
            $apiUser=$this->apiUserModel->get($params['appid']);
            $paramsNeedSign['sign']=$this->md5SignLib->getSign($paramsNeedSign,$apiUser->secret);
        }else{
            $params=['appid'=>'','sign'=>'','timestamp'=>'','nonce_str'=>''];
            foreach ($params as $k=>$v){
                $params[$k]=$request->param($k);
                if (empty($params[$k])){
                    throw new \Exception('param "'.$k.'" should not be empty');
                }
            }
        }

        $apiUser=$this->apiUserModel->get($params['appid']);
        if (empty($apiUser)){
            throw new \Exception('错误的appid');
        }

        try{
            $this->md5SignLib->verifySign($paramsNeedSign,$apiUser->secret);
        }catch (\Exception $e){
            if (env('app_env')=='dev'){
                $this->return['presign_str']=$this->md5SignLib->getLastPresignStr();
            }
            throw $e;
        }

        if (abs((time()-$params['timestamp']))>300){
            throw new \Exception('error timestamp');
        }
        #权限控制
        #权限节点
        $api=strtolower(config('api.server').'/'.$request->controller().'/'.$request->action());

        if (!$this->apiAuthModel->hasAuth($apiUser->appid,$api)){
            throw new \Exception('没有接口权限:'.$api);
        }

    }

    private function isSandBox(){
        if(request()->param('sandbox')=='heyu'||request()->ip()=='127.0.0.1'){
            return true;
        }else{
            return false;
        }
    }

}